#include "SDFileSystem.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "pen4v3b.h"
#include "pen4v3b_str.h"

extern SDFileSystem sd;
extern Serial pc;

//**********************************************************
//　関節パラメータファイルを読み込む
//　引数
//　　filename0：関節パラメータファイルのファイル名
//　　leg_prm：脚用構造体
//　　arm_prm：腕用構造体
//　　body_prm：体用構造体
//　戻り値
//　　　０：成功
//　　－１：失敗
//**********************************************************
int body_prm_read(char filename0[32], struct LEG_PRM leg_prm[2]
                  , struct ARM_PRM arm_prm[2], struct BODY_PRM *body_prm)
{
int     i, j, k, l;
char    line[256],fields[32][64];

    //ファイルを開く
    FILE *fp = fopen(filename0, "rt");
    if(fp == NULL) {
        pc.printf("Error! File Not Found!(body_prm_read)\n");
        return(-1);
    }

    //kは脚番号、0:右足　1:左足
    for(k = 0; k < 2; k++)
    {
        fgets(line, 256, fp);//コメント読み飛ばし
        //脚骨格の長さを読み込む
        for(i = 0; i < 2; i++)
        {
            fgets(line, 256, fp);
            devide_into_field(line, ",", fields);
            leg_prm[k].lbone_length[i] = atof(fields[1]);
        }

        //腕骨格の長さを読み込む
        for(i = 0; i < 3; i++)
        {
            fgets(line, 256, fp);
            j = devide_into_field(line, ",", fields);
            arm_prm[k].bone_length[i] = atof(fields[1]);
        }

        //サーボ角度－出力データ対応関係の読み込み
        //脚
        for(i = 0; i < 2; i++)
        {
            fgets(line, 256, fp);
            devide_into_field(line, ",", fields);
            for(l = 0; l < 6; l++)
                leg_prm[k].servo_st_en[i][l] = atof(fields[l + 1]);
        }
        //腕
        for(i = 0; i < 3; i++)
        {
            fgets(line, 256, fp);
            devide_into_field(line, ",", fields);
            for(l = 0; l < 6; l++)
                arm_prm[k].servo_st_en[i][l] = atof(fields[l + 1]);
        }
    }

    fgets(line, 256, fp);//コメント読み飛ばし
    //胴体のデータ読み込み
    //サーボ角度－出力データ対応関係の読み込み
    for(i = 0; i < 3; i++)
    {
        fgets(line, 256, fp);
        devide_into_field(line, ",", fields);
        for(l = 0; l < 6; l++)
                body_prm->servo_st_en[i][l] = atof(fields[l + 1]);
    }

    //ファイルを閉じる
    fclose(fp);
    return(0);
}

//**********************************************************
//delimiterで指示されたキャラクターでラインを分割する
//例：  1,  2,  3,  :abcde
//  delimiter = ":" の時
//  fields[0][] = "  1,  2,  3"
//  fields[1][] = "abcde"
//　引数
//　　line：分割対象の文字列
//　　delimiter：区切り記号
//　　fields：分割後の文字列
//　戻り値
//　　分割後の要素数
//**********************************************************
int devide_into_field(char line[1024], char delimiter[16], char fields[32][64])
{
int     i,j,l,m,del_num,length;
int     flg;

    //文字列の長さを求める
    length = (int)strlen(line);
    del_num = (int)strlen(delimiter);
    l = 0;
    m = 0;
    //分割処理
    for(i = 0; i < length; i++)
    {
      fields[l][m] = line[i];
      flg = 0;
      for(j = 0; j < del_num; j++)
      {
      if((line[i] == delimiter[j] || i == length - 1) && flg == 0)
        {
          if(i != length - 1) fields[l][m] = 0;
          else                fields[l][m + 1] = 0;
          l++;
          if(l > 32)
          {
              pc.printf("Error! Out of array(devide_into_field)\n");
              exit(0);
          }
          m = -1;
          flg = 1;
        }
      }
      m++;
    }
    return(l);
}

