#include <stdio.h>
//#include <conio.h>
#include <math.h>
#include "pen4v3b.h"
#include "pen4v3b_str.h"

//**********************************************************
//　足先座標からＲＣサーボの出力データを求める
//　引数
//　　coo：足先基準点の座標、単位はｍｍ
//    coo[0][0]：右足のＸ座標
//    coo[0][1]：右足のＹ座標
//    coo[0][2]：右足のＺ座標
//    coo[1][0]：左足のＸ座標
//    coo[1][1]：左足のＹ座標
//    coo[1][2]：左足のＺ座標
//
//　　angle：関節の角度、単位はラジアン
//    angle[0][0]：右脚の０番関節の角度
//    angle[0][1]：右脚の１番関節の角度
//    angle[1][0]：左脚の２番関節の角度
//    angle[1][1]：左脚の３番関節の角度
//
//　　q_servo：ＲＣサーボ出力値
//    q_servo[0][0]：右脚の０番サーボのデータ
//    q_servo[0][1]：右脚の１番サーボのデータ
//    q_servo[1][0]：左脚の２番サーボのデータ
//    q_servo[1][1]：左脚の３番サーボのデータ
//
//　　leg_prm：脚の関節パラメータ用構造体
//　戻り値
//　　　０：成功
//　　－１：失敗
//**********************************************************
int coo2servo(double coo[2][3], double angle[2][2], int q_servo[2][2], struct LEG_PRM leg_prm[2])
{
int     i, j, k, err_flg;
double  coo_work[3][2], coo_buff[2][3], link[2];
double  x, y, leg_sign;
double  foot_uX, foot_uZ;

Calc_2D     calc_2D;

    leg_sign = 1.0;//右脚から計算を開始する
    for(i = 0; i < 2; i++)
    {
        //元の座標を保存しておく
        for(j = 0; j < 3; j++) coo_buff[i][j] = coo[i][j];

        //特異点が与えられた時の例外処理
        //X=0, Y=0, Z=0は特異点なので、もしも入力された時は少しずらす
        if(coo[i][0] == 0.0) coo[i][0] = 0.0001;

        //ヒザの座標を求める計算開始
        //足首関節の位置を計算する。座標原点は股関節軸
        foot_uX = coo[i][0];
        foot_uZ = coo[i][2] - (leg_prm[i].lbone_length[0] + leg_prm[i].lbone_length[1]);
        //股関節の座標を２円の交点計算用ワーク変数にセットする
        coo_work[0][0] = 0.0;
        coo_work[0][1] = 0.0;
        //足首関節の座標をを２円の交点計算用ワーク変数にセットする
        coo_work[1][0] = foot_uX;
        coo_work[1][1] = foot_uZ;
        //リンクの長さを２円の交点計算用ワーク変数に、円の半径としてセットする
        link[0] = leg_prm[i].lbone_length[0];
        link[1] = leg_prm[i].lbone_length[1];
        //２円の交点（ヒザの座標）を計算する
        if(calc_2D.crcir(coo_work, link) == -1)
        {
            //２円が交わらなかった場合のエラー処理
            printf("Out range of the leg(coo2servo : bone calk : knee)\n");
            printf("Leg number : %d\n Coordinate : ", i);
            for(k = 0; k < 3; k++)
            {
                printf("%5.1f,", coo_buff[i][k]);
            }
            printf("\n");
            //printf("何かキーを押してください\n");
            //getch();
            return(-1);
        }
        //1番関節の角度を求める（中立位置からの角度）
        x = -coo_work[0][1];
        y = coo_work[0][0];
        angle[i][1] = atan2(y, x) * leg_sign;

        //0番関節の角度を求める（中立位置からの角度）
        x = -(foot_uZ - coo_work[0][1]);
        y = foot_uX - coo_work[0][0];
        angle[i][0] = atan2(y, x) * leg_sign;

        //角度データ（ラジアン）をサーボデータに変換
        for(k = 0; k < 2; k++)
        {
            q_servo[i][k] = angle2servo(k, angle[i][k], leg_prm[i]);
        }
        //入力座標値をワーク変数から復帰
        for(j = 0; j < 3; j++) coo[i][j] = coo_buff[i][j];
        //計算対象を右脚から左脚に切り替える
        leg_sign = -1.0;
    }

    //計算結果がＲＣサーボの最大可動範囲を超えていないかチェック
    err_flg = 0;
    for(i = 0; i < 2; i++)
    {
        for(j = 0; j < 2; j++)
        {
            if(q_servo[i][j] > 4000 || q_servo[i][j] < 320)
            {
                printf("Servo limit exceed!(coo2servo)\n");
                printf("Leg number : %d , Joint Number%d \n", i, j);
                err_flg = 1;
                for(k = 0; k < 2; k++)
                    printf("%5.1f, ", coo_buff[i][k]);
                printf("\n");
            }
        }
    }
    if(err_flg == 1)
    {
        for(i = 0; i < 2; i++)
        {
            for(j = 0; j < 2; j++)
            {
                printf("%3d, ", q_servo[i][j]);
            }
        }
        //_getch();
        return(-1);
    }
return(0);
}

//**********************************************************
//関節の角度を直線補間でＲＣサーボへの出力データに変換する。
//脚関節用
//　引数
//　　joint_num：関節番号　０：ヒザの下側　１：ヒザの上側
//　　joint_angle：関節角度
//　　leg_prm：脚の関節パラメータ用構造体
//　戻り値
//　　ＲＣサーボ出力データ
//**********************************************************
int angle2servo(int joint_num, double joint_angle, struct LEG_PRM leg_prm)
{
int     i, j, servo_data;
double  a, b, q_refp[3];

    j = 0;
    for(i = 0; i < 3; i++)
    {
        j = i * 2;
        q_refp[i] = leg_prm.servo_st_en[joint_num][j] / 180.0 * PI;
    }

    //補間直線の傾きを求める
    if(joint_angle > q_refp[1])
        a = (leg_prm.servo_st_en[joint_num][5] - leg_prm.servo_st_en[joint_num][3])
            / (q_refp[2] - q_refp[1]);
    else
        a = (leg_prm.servo_st_en[joint_num][3] - leg_prm.servo_st_en[joint_num][1])
            / (q_refp[1] - q_refp[0]);

    //補間直線のＹ切片を求める
    b = leg_prm.servo_st_en[joint_num][3] - a * q_refp[1];
    
    servo_data = (int)(a * joint_angle + b);

    return(servo_data);
}

//**********************************************************
//関節の角度を直線補間でＲＣサーボへの出力データに変換する。
//首、腰関節用
//　引数
//　　joint_num：関節番号　０：腰ピッチ軸　１：腰ロール軸　２：首
//　　joint_angle：関節角度
//　　body_prm：腰の関節パラメータ用構造体
//　戻り値
//　　ＲＣサーボ出力データ
//**********************************************************
int angle2servo2(int joint_num, double joint_angle, struct BODY_PRM body_prm)
{
int     i, j, servo_data;
double  a, b, q_refp[3];

    j = 0;
    for(i = 0; i < 3; i++)
    {
        j = i * 2;
        q_refp[i] = body_prm.servo_st_en[joint_num][j] / 180.0 * PI;
    }

    //補間直線の傾きを求める
    if(joint_angle > q_refp[1])
        a = (body_prm.servo_st_en[joint_num][5] - body_prm.servo_st_en[joint_num][3])
            / (q_refp[2] - q_refp[1]);
    else
        a = (body_prm.servo_st_en[joint_num][3] - body_prm.servo_st_en[joint_num][1])
            / (q_refp[1] - q_refp[0]);

    //補間直線のＹ切片を求める
    b = body_prm.servo_st_en[joint_num][3] - a * q_refp[1];
    
    servo_data = (int)(a * joint_angle + b);

    return(servo_data);
}

//**********************************************************
//関節の角度を直線補間でＲＣサーボへの出力データに変換する。
//腕関節用
//　引数
//　　joint_num：関節番号　０：肩ロール軸　１：肩ピッチ軸　２：ヒジ
//　　joint_angle：関節角度
//   arm_prm: structure fot arm joint
//　戻り値
//　　ＲＣサーボ出力データ
//**********************************************************
//腕関節用
int angle2servo3(int joint_num, double joint_angle, struct ARM_PRM arm_prm)
{
int     i, j, servo_data;
double  a, b, q_refp[3];

    j = 0;
    for(i = 0; i < 3; i++)
    {
        j = i * 2;
        q_refp[i] = arm_prm.servo_st_en[joint_num][j] / 180.0 * PI;
    }

    //補間直線の傾きを求める
    if(joint_angle > q_refp[1])
        a = (arm_prm.servo_st_en[joint_num][5] - arm_prm.servo_st_en[joint_num][3])
            / (q_refp[2] - q_refp[1]);
    else
        a = (arm_prm.servo_st_en[joint_num][3] - arm_prm.servo_st_en[joint_num][1])
            / (q_refp[1] - q_refp[0]);

    //補間直線のＹ切片を求める
    b = arm_prm.servo_st_en[joint_num][3] - a * q_refp[1];
    
    servo_data = (int)(a * joint_angle + b);
    
    if(servo_data < 0 || servo_data > 3720) printf("Worning! Arm joints limit over!(angle2servo3)\n");

    return(servo_data);
}