#include <stdio.h>
//#include <conio.h>
#include <math.h>
#include "pen4v3b.h"
#include "pen4v3b_str.h"

//**********************************************************
//　手先座標からＲＣサーボの出力データを求める
//　引数
//　　coo：足先基準点の座標、単位はｍｍ
//    coo[0][0]：右手のＸ座標
//    coo[0][1]：右手のＹ座標
//    coo[0][2]：右手のＺ座標
//    coo[1][0]：左手のＸ座標
//    coo[1][1]：左手のＹ座標
//    coo[1][2]：左手のＺ座標
//
//　　angle：関節の角度、単位はラジアン
//    angle[0][0]：右手の０番関節の角度
//    angle[0][1]：右手の１番関節の角度
//    angle[0][2]：右手の２番関節の角度
//    angle[1][0]：左手の０番関節の角度
//    angle[1][1]：左手の１番関節の角度
//    angle[1][2]：左手の２番関節の角度
//
//　　q_servo：ＲＣサーボ出力値
//    q_servo[0][0]：右手の０番サーボのデータ
//    q_servo[0][1]：右手の１番サーボのデータ
//    q_servo[0][2]：右手の２番サーボのデータ
//    q_servo[1][0]：左手の０番サーボのデータ
//    q_servo[1][1]：左手の１番サーボのデータ
//    q_servo[1][2]：左手の２番サーボのデータ
//
//　　arm_prm：手の関節パラメータ用構造体
//　戻り値
//　　　０：成功
//　　－１：失敗
//**********************************************************
int coo2servo2(double coo[2][3], double angle[2][3], int q_servo[2][3], struct ARM_PRM arm_prm[2])
{
int     i, j, k, err_flg;
double  coo_work[3][2], coo_buff[2][3], link[2];
double  x, y, arm_sign, offset_ang;
double  q0, q2, x0, y0, z0, x01, y01, z01, x21, y21, z21;

Calc_2D     calc_2D;

    arm_sign = 1.0;//右手から計算を開始する
    offset_ang = -17.7747 / 180.0 * PI;//右手（肩ピッチ軸）のオフセット角度・・・組み立て図参照
    //備考：関節に振ったアルファベット等は、手書きメモ参照
    //腕の座標軸の方向も脚と同じ
    for(i = 0; i < 2; i++)
    {
        x0 = coo[i][0];
        y0 = coo[i][1];
        z0 = coo[i][2];

        //肩のロール軸の角度を求める
        q0 = atan2(y0, -z0);
        angle[i][0] = q0;

        //点Ｂ，Ｃ，Ｄ（肩から先の関節）をＸＺ平面に射影する
        //B1の座標
        x21 = 0.0;
        y21 = 0.0;
        z21 = -arm_prm[i].bone_length[0];

        //D1の座標（Ｘ軸の周りに点Ｄ(手先)を-q0回転した結果のＸＺ平面上の点）
        x01 = x0;
        y01 = 0.0;
        z01 = -sqrt(y0 * y0 + z0 * z0);

        //ＸＺ平面上に射影されたヒジの座標を求める計算開始
        //座標原点は肩関節軸のピッチ軸
        //股関節の座標を２円の交点計算用ワーク変数にセットする
        coo_work[0][0] = z21;
        coo_work[0][1] = 0.0;

        //足首関節の座標をを２円の交点計算用ワーク変数にセットする
        coo_work[1][0] = z01;
        coo_work[1][1] = x01;

        //リンクの長さを２円の交点計算用ワーク変数に、円の半径としてセットする
        link[0] = arm_prm[i].bone_length[1];
        link[1] = arm_prm[i].bone_length[2];

        //２円の交点（ヒジの座標）を計算する
        if(calc_2D.crcir(coo_work, link) == -1)
        {
            //２円が交わらなかった場合のエラー処理
            printf("手が届かない範囲を指示されました(coo2servo2：骨格計算：ヒジ)\n");
            printf("手番号 : %d\n 座標:", i);
            for(k = 0; k < 3; k++)
            {
                printf("%5.1f,", coo_buff[i][k]);
            }
            printf("\n");
            printf("何かキーを押してください\n");
            //_getch();
            return(-1);
        }
        //肩ピッチ軸の角度を求める（中立位置からの角度）
        x = -coo_work[0][0];
        y = coo_work[0][1];
        angle[i][1] = (atan2(y, x) + offset_ang) * arm_sign;

        //ヒジ関節の角度を求める（中立位置からの角度）
        coo_work[0][0] = z21;
        coo_work[0][1] = 0.0;

        coo_work[1][0] = -x;
        coo_work[1][1] = y;

        coo_work[2][0] = z01;
        coo_work[2][1] = x01;

        q2 = calc_2D.vector_dir_f(coo_work);
        angle[i][2] = (-(q2 + PI / 2.0)) * arm_sign;

        //角度データ（ラジアン）をサーボデータに変換
        for(k = 0; k < 3; k++)
        {
            q_servo[i][k] = angle2servo3(k, angle[i][k], arm_prm[i]);
        }
        //計算対象を右手から左手に切り替える
        arm_sign = -1.0;
    }

    //計算結果がＲＣサーボの最大可動範囲を超えていないかチェック
    err_flg = 0;
    for(i = 0; i < 2; i++)
    {
        for(j = 0; j < 3; j++)
        {
            if(q_servo[i][j] > 4000 || q_servo[i][j] < 320)
            {
                printf("サーボ出力値がオーバー／アンダーフローしました(coo2servo)\n");
                printf("手番号: %d 、関節番号 %d \n", i, j);
                err_flg = 1;
                for(k = 0; k < 2; k++)
                    printf("%5.1f, ", coo_buff[i][k]);
                printf("\n");
            }
        }
    }
    if(err_flg == 1)
    {
        for(i = 0; i < 2; i++)
        {
            for(j = 0; j < 3; j++)
            {
                printf("%3d, ", q_servo[i][j]);
            }
        }
        return(-1);
    }
return(0);
}

