#include "mbed.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "pen4v3b.h"
#include "pen4v3b_str.h"

extern DigitalOut led1;

extern Serial pc;
extern Serial xbee;

//**********************************************************
// Get text files from host PC
//**********************************************************
void get_text_files(Serial *port)
{
    char    work_c,filename0[32],work_s[64][128];
    unsigned int file_num;
    int     i, j, k, line_num;

    //get file number to get
    while (!port->readable());
    file_num = port->getc();

    for(i = 0; i < file_num; i++) {
        //get file name to receive.
        j = 0;
        do {
            while (!port->readable());
            work_c = port->getc();
            filename0[j] = work_c;
            j++;
        } while(work_c != 0x0a);
        filename0[j] = 0;

        FILE *fpo2 = fopen(filename0 , "wt");
        line_num = 0;
        if (fpo2 != NULL) {
            do {
                //get line
                j = 0;
                do {
                    while (!port->readable());
                    work_c = port->getc();
                    work_s[line_num][j] = work_c;
                    j++;
                } while(work_c != 0x0a && work_c != 0x1a);
                work_s[line_num][j] = 0;
                line_num++;
            } while(work_c != 0x1a);
            for(k = 0; k < line_num - 1; k++) {
                fprintf(fpo2, "%s", work_s[k]);
            }
            fclose(fpo2);
        } else {
            //printf("Error! cannot create the file.(get_text_files)\n");
            led1 = 1;
            return;
        }
    }
    return;
}


//**********************************************************
//　足先軌道点列をファイルに書き出す
//　引数
//　　filename0：関節パラメータファイルのファイル名
//　　coo_wav1：足先軌道点列
//　　num：足先軌道点列のデータ数
//　戻り値
//　　　０：成功
//　　－１：失敗
//**********************************************************
int coo_fwrite(char filename0[32], double coo_wav1[2][WAVE_MAX][3], int num)
{
    int     i,j,k;

    FILE *fpo = fopen(filename0 , "wt");
    if (fpo != NULL) {
        for(i = 0; i < num; i++) {
            for(j = 0; j < 2; j++) {
                for(k = 0; k < 3; k++) {
                    fprintf(fpo, "%f, ", coo_wav1[j][i][k]);
                }
            }
            fprintf(fpo, "\n");
        }
        fclose(fpo);
    } else {
        printf("Error. Cannot create foot points coordinate file.(coo_fwrite)\n");
        return(-1);
    }
    return(0);
}

//**********************************************************
//**********************************************************
int savo_fwrite(char filename0[32], int servo_data[WAVE_MAX][12], int num)
{
    int     i,j, debug;

    if(num >= 64) printf("Worning : Motion data element number exceed the limit 64.\n");

    //Print servo data csv text if "debug" flug is set to be "1" .
    debug = 0;

    FILE *fpo2 = fopen(filename0 , "wt");
    if (fpo2 != NULL) {
        for(j = 0; j < num; j++) {
            for(i = 0; i < 11; i++) {
                fprintf(fpo2, "%d,", servo_data[j][i]);
            }
            fprintf(fpo2, "%d,", servo_data[j][i]);
            fprintf(fpo2, "\n");

            if(debug == 1) {
                for(i = 0; i < 11; i++) {
                    printf("%d,", servo_data[j][i]);
                }
                printf("%d,", servo_data[j][i]);
                printf("\n");
            }
        }
        fclose(fpo2);
    } else {
        printf("Error! cannot create the servo data file.(savo_fwrite)\n");
        return(-1);
    }
    return(0);
}
//**********************************************************
//モーションデータをファイルから読み込む
//**********************************************************
int savo_fread(char filename[32], int servo_data[WAVE_MAX][12])
{
    int     i, j;
    char    buff[256], fields[32][64];
    FILE    *fp;

    if ((fp = fopen(filename , "rt")) == NULL) {
        printf("Motion data file open error!(savo_fread)\n");
        return(-1);
    }

    i = 0;
    while (fgets(buff, 256, fp) != NULL) {
        devide_into_field(buff, "," , fields);
        for(j = 0; j < 12; j++) servo_data[i][j] = atoi(fields[j]);
        i++;
    }
    fclose(fp);
    return(i);
}

