#include <stdio.h>
#include <math.h>
#include "pen4v3b.h"

/************************************************************************/
/*   ２円の交点の座標を求める                                           */
/*   coo[2][2]...２円の中心座標, 結果の交点座標もここへ返される　       */
/*   r[2]     ...２円の半径                                             */
/************************************************************************/
int Calc_2D::crcir(double coo[2][2],double r[2])
{
int     ii,jj;
double   x0,y0,x5,y5;
double   a,aa,b,e,f,h,i,l,l2,m,o,p;
double  a1,d;
double   vr0[2],vx[2],vy[2],r0,r1,r2,r3,cosq1,sinq1;
double   offset,offset1;


    //２円のX座標が同じ時の例外処理
    if(coo[0][0] == coo[1][0])
    {
        x0 = coo[0][0];
        y0 = coo[0][1];
        //x1 = coo[1][0];
        //y1 = coo[1][1];

        d = coo[1][1] - coo[0][1];
        a1 = ((r[0] * r[0]) - (r[1] * r[1]) + (d * d)) / (2 * d);
        aa = (r[0] * r[0]) - (a1 * a1);
        if(aa < 0.0)
        {
            //printf("２円の交点が求まりません(crcir)\n");
            return(-1);
        }
        b = sqrt(aa);

        coo[0][0] = x0 - b;
        coo[0][1] = y0 + a1;

        coo[1][0] = x0 + b;
        coo[1][1] = y0 + a1;

        return(0);
    }

    offset1 = 0.0;
    for(ii = 0; ii < 2; ii++)
    {
        for(jj = 0; jj < 2; jj++)
        {
            if(coo[ii][jj] < offset1) offset1 = coo[ii][jj];
        }
    }

    offset = (r[0] + r[1]) * 2 - offset1;

    h = coo[0][0] + offset;
    i = coo[0][1] + offset;
    e = coo[1][0] + offset;
    f = coo[1][1] + offset;
    r1 = r[0];
    r2 = r[1];
    r3 = r1 + r2;

    vr0[0] = e - h;
    vr0[1] = f - i;
    r0 = sqrt( vr0[0] * vr0[0] + vr0[1] * vr0[1]);

    /*まず交点が作成可能かチェック、ダメならエラー*/
    if ( r0 > r3)
    {
        return(-1);
    }

    cosq1 = (r0 * r0 + r1 * r1 - r2 * r2) / ( 2 * r0 * r1 );
    sinq1 = sqrt(1 - cosq1 * cosq1);

    vx[0] = vr0[0] / r0;
    vx[1] = vr0[1] / r0;
    vy[0] = -vx[1];
    vy[1] = vx[0];

    l = r1 * cosq1 * vx[0] + r1 * sinq1 * vy[0] + h;
    m = r1 * cosq1 * vx[1] + r1 * sinq1 * vy[1] + i;

    coo[0][0] = l - offset;
    coo[0][1] = m - offset;

    a = vx[1] / vx[0];
    b = vy[1] / vy[0];
    x5 = (a * h - b * l + m - i) / (a - b);
    y5 = a * (x5 - h) + i;
    a = x5 - l;
    b = y5 - m;
    l2 = sqrt( a * a + b * b);
    o = - vy[0] * 2 * l2 + l;
    p = - vy[1] * 2 * l2 + m;
    coo[1][0] = o - offset;
    coo[1][1] = p - offset;

return(0);
}
/*******************************************************************/
/**  任意の２点間の距離を求める                                   **/
/**  coo[0][0] : x1  coo[0][1] : y1                               **/
/**  coo[1][0] : x2  coo[1][1] : y2                               **/
/*******************************************************************/
double Calc_2D::dist_2points(double coo[2][2])
{
double distance;
    distance = sqrt(pow((coo[0][0]-coo[1][0]),2) + pow((coo[0][1]-coo[1][1]),2));
    return(distance);
}

/*******************************************************************/
/**  回転の１次変換で任意の点を中心にして任意の点を回転する       **/
/**  coo[0][0] : x1  coo[0][1] : y1                               **/
/**  coo[1][0] : x2  coo[1][1] : y2                               **/
/**  canter_x, y : 回転変換の中心　                               **/
/**  direction   : 回転角　　　　　　                             **/
/**  data_num    : 回転するデータ数                               **/
/**  　　cooに、２次元配列の先頭を指すポインタを渡す              **/
/*******************************************************************/
void Calc_2D::rotate_1D(double coo[WAVE_MAX][2], double center_x,
            double center_y, double direction, int data_num)
{
int     i;
double   x,y;

/* 受け取った座標群を回転の中心が原点に来るように平行移動する */
   for (i = 0; i < data_num; i++)
   {
     coo[i][0] = coo[i][0] - center_x;
     coo[i][1] = coo[i][1] - center_y;
   }

/* 原点を中心として回転の一次変換をする */
   for (i = 0; i < data_num; i++)
   {
     x = coo[i][0];
     y = coo[i][1];
     coo[i][0] = x * cos (direction) - y * sin (direction);
     coo[i][1] = x * sin (direction) + y * cos (direction);
   }

/* 受け取った座標群を回転の中心が元の位置に来るように平行移動し戻す */
   for (i = 0; i < data_num; i++)
   {
     coo[i][0] = coo[i][0] + center_x;
     coo[i][1] = coo[i][1] + center_y;
   }
}

/******************************************************************/
/**  隣接するベクトルの一方からみた他方の角度を計算する          **/
/**  coo : 計算したいベクトルの座標　　　　　　　　　　          **/
/**  ベクトルはcoo[0][x]->coo[1][x]->coo[2][x]の順に接続するので **/
/**  実際には coo[1][x] での角度が求められる。　　　　           **/
/******************************************************************/
double Calc_2D::vector_dir_f(double coo[3][2])
{
double   coof[2][2],q1,q3;

    /* make vector BC*/
    coof[0][0] = coo[2][0] - coo[1][0];
    coof[0][1] = coo[2][1] - coo[1][1];

    /* make vector AB*/
    coof[1][0] = coo[1][0] - coo[0][0];
    coof[1][1] = coo[1][1] - coo[0][1];


    /* cluculate Q1 */
    q1 = atan2(coof[1][1], coof[1][0]);

    /*************************************/
    /* adjst Q2 to Q3 */
    rotate_1D(coof, 0.0, 0.0, -q1, 1);

    /*************************************/
    /* cluculate Q3 */
    q3 = atan2(coof[0][1], coof[0][0]);

return(q3);
}

/*******************************************************************/
/**  任意の２点間の距離を求める                                   **/
/**  coo[0][0] : x1  coo[0][1] : y1  coo[0][2] : z1               **/
/**  coo[1][0] : x2  coo[1][1] : y2  coo[1][2] : z1               **/
/*******************************************************************/
double Calc_3D::dist_2points3D(double coo[2][3])
{
double distance;
    distance = sqrt(pow((coo[0][0]-coo[1][0]),2) + pow((coo[0][1]-coo[1][1]),2) + pow((coo[0][2]-coo[1][2]),2));
    return(distance);
}

