#include <stdio.h>
#include <math.h>
#include "pen4v3b.h"
#include "pen4v3b_str.h"

//**********************************************************
//　数直線を作成する
//　区間の速度はｃｏｓ関数で変化する
//　数直線は０から始まる
//　引数
//　　coo：計算結果の数直線
//　　dist：数直線の長さ
//　　q_wave：脚のモーションデータ
//　　data_num：数直線の分割数
//　　q_wave：脚のモーションデータ
//　　leg_prm：脚用構造体
//　戻り値
//　　　０：成功
//　　－１：失敗
//**********************************************************
int num_line_1Da(double coo[WAVE_MAX], double dist, int data_num)
{
int     i;
double  a, t, dt_rad;
double  v;

    if(data_num > (int)WAVE_MAX)
    {
        printf("Error! Out of array(num_line_1Da)");
        return(-1);
    }

    dt_rad = 2.0 * PI / (double)data_num;
    a = dist / (double)data_num;

    coo[0] = 0.0;
    t = 0.0;

    for(i = 1; i < data_num; i++)
    {
          v = - (a * cos(t)) + a;
          coo[i] = coo[i - 1] + v;
          t = t + dt_rad;
    }

    return(data_num);
}

//**********************************************************
//　数直線を作成する
//　区間の速度はｃｏｓ関数で変化する
//　数直線は０から始まる
//　引数
//　　coo：計算結果の数直線
//　　　coo[][0]：積算位置
//　　　coo[][1]：差分
//　　number_line：数直線用構造体
//　戻り値
//　　データ数
//**********************************************************
int num_line1b(double coo[WAVE_MAX][2], struct NUMBER_LINE number_line)
{
int     i,num;
double  dist;
double  a, t, dt_rad;
double  v;

    dist = number_line.length;

    num = number_line.point_num;
    if(num > (int)WAVE_MAX)
    {
        printf("Error! Out of array(num_line1b)");
        return(-1);
    }

    dt_rad = 2.0 * PI / (double) number_line.point_num;
    t = dt_rad;
    a = dist / (double)number_line.point_num;

    coo[0][0] = 0.0;
    coo[0][1] = 0.0;
    for(i = 1; i < num; i++)
    {
          v = - (a * cos(t)) + a;

          coo[i][0] = coo[i - 1][0] + v;//積算位置
          coo[i][1] = v;//差分

          t = t + dt_rad;
    }
    return(num);
}
