#include "mbed.h"
#include "SDFileSystem.h"
#include <stdio.h>
#include "pen4v3b.h"
#include "pen4v3b_str.h"
#include "pen4v3b_class.h"

SDFileSystem sd(p5, p6, p7, p8, "sd");

DigitalOut led1(LED1);
DigitalOut led2(LED2);
DigitalOut led3(LED3);
DigitalOut led4(LED4);

DigitalIn dip_sw1(p11);
DigitalIn dip_sw2(p12);
DigitalIn pwm_status(p22);

AnalogIn Battery(p15);

PwmOut gyro_ref_pwm(p21);
Timer timer;

Serial pc(USBTX, USBRX); // tx, rx 9600bps
Serial xbee(p13, p14); // tx, rx 9600bps default, Port connected to Xbee s2
Serial h8(p9, p10); // tx, rx 9600bps default, Port connected to H8/3664

unsigned int read_ii(Serial *port);
void print_ii(Serial *port, unsigned int data);
void get_text_files(Serial *port);

int main()
{
    int     num, dip_sw;
    int     i, data_num;
    int     q_servo[2][2];
    int     q_servo2[2][3];
    int     servo_data_single[12];
    int     gyro_newtral[2];
    
    double  coo[2][3];
    double  angle[2][2];
    double  arm_ang[2][3];

    struct  LEG_PRM leg_prm[2];
    struct  ARM_PRM arm_prm[2];
    struct  BODY_PRM body_prm;

    struct  STEP_INF step_inf;
    int     servo_wav2[WAVE_MAX][12];

    class   Walk02 walk02;

    unsigned char command;
    struct SIO sio;

    pc.baud(38400);
    h8.baud(38400);
    xbee.baud(57600);

    gyro_ref_pwm.period_ms(20);
    //gyro_ref_pwm.period_ms(1000);
    gyro_ref_pwm = 1.5 / 20.0;

    gyro_newtral[0] = 3000;
    gyro_newtral[1] = 3000;

    dip_sw = 0;
    //dip_sw1&2 is active low.
    if (dip_sw1 == 0) dip_sw = dip_sw + 1;
    if (dip_sw2 == 0) dip_sw = dip_sw + 2;

    //関節パラメータファイル読み取り
    if(body_prm_read("/sd/pen4_pal.csv", leg_prm, arm_prm, &body_prm) == -1) {
        pc.printf("Joint palameter file read error!\n");
    }

    switch(dip_sw) {
        case 0:
            //作業選択メニュー表示
            num = 0;
            coo[0][0] =  -4.0;// X(右足)
            coo[0][1] =   0.0;// Y
            coo[0][2] =   3.0;// Z

            coo[1][0] =  -4.0;// X(左足)
            coo[1][1] =   0.0;// Y
            coo[1][2] =   3.0;// Z

            while (!pc.writeable());
            sio.Battery = Battery * 3.3 * 3.0;
            printf("\n");
            printf("Battery : %5.1f (V)\r\n",sio.Battery);

            while(num != 99) {
                printf("\n");
                printf("Select number.\n");

                printf(" 1:Resume foot position to the home.\n");

                printf(" 2:Get Text files.\n");

                printf(" 3:Get gyro newtral\n");

                printf(" 4:Body direction adjust test\n");

                printf(" 5:Motion play.\n");

                printf(" 6:Gyro amp test.\n");

                printf(" 7:Joypad test.\n");

                printf(" 8:Arm rev. kine. calc. test.\n");

                printf("99:Exit loop\n");

                //作業番号をキーボードから読み取る
                scanf ("%d",&num);
                while (!pc.readable());
                i = pc.getc();//dummy LF read

                printf("%d\n", num);
                /*******************************************************/
                //逆運動学計算関数のテスト
                if(num == 1) {
                    //腕関節位置を初期化
                    arm_ang[0][0] = -15.0 / 180.0 * PI;// SV6
                    arm_ang[0][1] = -35.0 / 180.0 * PI;// SV7
                    arm_ang[0][2] =  70.0 / 180.0 * PI;// SV8
                    arm_ang[1][0] =  15.0 / 180.0 * PI;// SV9
                    arm_ang[1][1] =  35.0 / 180.0 * PI;// SV10
                    arm_ang[1][2] = -70.0 / 180.0 * PI;// SV11

                    //ヒザの屈伸　歩行開始と歩行終了時のモーション生成
                    //足位置のオフセットをセット
                    //（左右の脚の長さに大きな誤差が合った時はここに数値を入れて調節する）
                    step_inf.offset[0][0]   =  0.0;//x right
                    step_inf.offset[1][0]   =  0.0;//x left
                    step_inf.offset[0][1]   =  0.0;//y right
                    step_inf.offset[1][1]   =  0.0;//y left
                    step_inf.offset[0][2]   =  0.0;//z right
                    step_inf.offset[1][2]   =  0.0;//z left

                    //腰を落とす距離をセット
                    step_inf.init_depth     = 3.0;
                    //腰を落とした時のＸ方向（前後方向）オフセット距離をセット
                    //ヒザを曲げた時にヒザ関節への負荷が変わる事により、関節の正常偏差が
                    //変わり、重心が動いてしまう現象を補償するための数値をセット
                    step_inf.init_offset    = 4.0;

                    //足位置のオフセットをセット
                    step_inf.offset[0][0]   =-step_inf.init_offset;//x right
                    step_inf.offset[1][0]   =-step_inf.init_offset;//x left
                    step_inf.offset[0][1]   =  0.0;//y right
                    step_inf.offset[1][1]   =  0.0;//y left
                    step_inf.offset[0][2]   =  step_inf.init_depth;//z right
                    step_inf.offset[1][2]   =  step_inf.init_depth;//z left

                    //信地旋回のモーション生成
                    step_inf.swingL         =  16.0;
                    step_inf.swingR         = -16.0;
                    step_inf.v_diff         = 10.0;
                    step_inf.travel         = coo[0][0] - coo[1][0];
                    step_inf.overlap_num    = 0;
                    step_inf.point_num      = 17;

                    data_num = walk02.turnR_last2(servo_wav2, leg_prm, arm_prm, body_prm, &step_inf, arm_ang);
                    savo_fwrite("/sd/20.csv", servo_wav2, data_num);
                    play_motion3("/sd/20.csv",gyro_newtral);
                }
                /*******************************************************/
                //
                if(num == 2) {
                    pc.printf("get_text_files start.\r\n");
                    get_text_files(&pc);
                }
                /*******************************************************/
                //Testing
                if(num == 3) {
                    get_gyro_newtral(gyro_newtral);
                    printf("[0] = %d, [1] = %d", gyro_newtral[0], gyro_newtral[1]);
                }
                /*******************************************************/
                //
                if(num == 4) {
                    if(coo[0][0] < 15.0 - 4.0) {
                        coo[0][0] = coo[0][0] + 2; // Right foot X coordinate.
                        coo[1][0] = coo[1][0] - 2; // Left foot X coordinate.
                    } else {
                        printf("Strok limit over!\n");
                        coo[0][0] = -4.0;
                        coo[1][0] = -4.0;
                    }

                    i = coo2servo(coo, angle, q_servo, leg_prm);

                    //Send the result if sucsess.
                    if(i == 0) {

                        //Get present positions for all the servos.
                        read_servo_data(servo_data_single);

                        //Over write the data for leg servos.
                        for(i = 0; i < 2; i++) {
                            servo_data_single[i] = q_servo[0][i];
                            servo_data_single[i+2] = q_servo[1][i];
                        }
                        update_servo_data(servo_data_single);
                    }
                    /*
                    if(coo[0][0] < 15.0 - 4.0) {
                        coo[0][0] = coo[0][0] + 2; // Right foot X coordinate.
                        coo[1][0] = coo[1][0] - 2; // Left foot X coordinate.
                    } else {
                        printf("Strok limit over!\n");
                        coo[0][0] = -4.0;
                        coo[1][0] = -4.0;
                    }
                    */
                    printf("coo[0][0] = %5.1f, coo[1][0] = %5.1f\n", coo[0][0], coo[1][0]);
                }
                /*******************************************************/
                //
                if(num == 5) {
                    get_gyro_newtral(gyro_newtral);
                    //stand lower
                    play_motion3("/sd/0.csv",gyro_newtral);
                    //first step
                    play_motion3("/sd/3.csv",gyro_newtral);
                    for(i = 0; i < 3; i++) {
                        play_motion3("/sd/2.csv",gyro_newtral);
                    }
                    //last step
                    play_motion3("/sd/4.csv",gyro_newtral);
                    //stand higher
                    play_motion3("/sd/1.csv",gyro_newtral);
                }
                /*******************************************************/
                //Testing
                if(num == 6) {
                    gyro_test(gyro_newtral);
                }
                /*******************************************************/
                //Testing
                if(num == 7) {
                    get_gyro_newtral(gyro_newtral);
                    joypad_test(gyro_newtral, leg_prm, arm_prm, body_prm);
                }
                /*******************************************************/
                /*******************************************************/
                //逆運動学計算関数のテスト
                if(num == 8) {
                    coo[0][0] =  10.0;// X(右手)
                    coo[0][1] = -20.0;// Y
                    coo[0][2] =-137.0;// Z

                    coo[1][0] =  10.0;// X(左手)
                    coo[1][1] =  20.0;// Y
                    coo[1][2] =-137.0;// Z

                    i = coo2servo2(coo, arm_ang, q_servo2, arm_prm);

                    //計算結果の表示（計算結果が求まった時のみ表示）
                    if(i == 0) {
                        for(i = 0; i < 2; i++) {
                            printf("Arm No. %d, data: %5d, %5d, %5d\n", i, q_servo2[i][0], q_servo2[i][1], q_servo2[i][2]);
                        }
                    }
                }
            }
        case 1:
            get_gyro_newtral(gyro_newtral);
            joypad_test(gyro_newtral, leg_prm, arm_prm, body_prm);

        default:
            pc.printf("Control program for Pen4v3 start.\r\n");
            while(1) {
                command = 0;
                if (pc.readable()) {
                    command = pc.getc();
                    sio_comm1(command, &sio);
                }
                /*
                if (xbee.readable()) {
                    worki = xbee.getc();
                    pc.putc(worki);
                }
                */
                if(pwm_status == 1) led1 = 1;
                else led1 = 0;
                //wait_ms(1);
            }
    }

}
