#include "mbed.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "pen4v3b.h"
#include "pen4v3b_str.h"
/*
#define  AMP0 0.20
#define  AMP1 0.15
*/
#define  AMP0 0.25
#define  AMP1 0.20
#define  DEC_RATE0 0.70
#define  DEC_RATE1 0.70


extern Serial h8;
extern Serial pc;
unsigned int read_ii(Serial *port);
void print_ii(Serial *port, unsigned int data);

void read_servo_data(int servo_data[12])
{
    int     i;

    //transmit command action to h8
    while (!h8.writeable());
    h8.putc('C');
    //get ack
    while (!h8.readable());
    h8.getc();
    //Servo data receive
    for (i = 0; i < 12; i++) {
        servo_data[i] = read_ii(&h8);
    }
}

void update_servo_data(int servo_data[12])
{
    int     i;

    //Gyro data dummy read **************************
    //transmit command action to h8
    while (!h8.writeable());
    h8.putc('B');
    //get ack
    while (!h8.readable());
    h8.getc();
    //gyro data receive
    for (i = 0; i < 2; i++) {
        read_ii(&h8);
    }
    //***********************************************
    //servo data send
    for (i = 0; i < 12; i++) {
        print_ii(&h8, servo_data[i]);
    }
}

//Get gyro newtral value.
void get_gyro_newtral(int gyro_newtral[2])
{
    int     i, j, av_count;
    unsigned int gyro_data[2];
    unsigned char work_c;

    gyro_newtral[0] = 0;
    gyro_newtral[1] = 0;
    av_count = 5;
    for(j = 0; j < av_count; j++) {
        //transmit command action to h8
        while (!h8.writeable());
        h8.putc('A');
        //get ack
        while (!h8.readable());
        work_c = h8.getc();

        //gyro data receive
        if(work_c == 'A') {
            for (i = 0; i < 2; i++) {
                gyro_data[i] = read_ii(&h8);
                gyro_newtral[i] = gyro_newtral[i] + (int)gyro_data[i];
            }
        } else {
            printf("Communication error @ get_gyro_newtral\n");
        }
    }
    //get average.
    for (i = 0; i < 2; i++) {
        gyro_newtral[i] = gyro_newtral[i] / av_count;
    }
}

//Play prefetched data from the motion data file.
void play_motion(int servo_data[WAVE_MAX][12], int data_num, int gyro_newtral[2])
{
    int     i, j, servo_data_work[2];
    unsigned int gyro_data[2];
    float amp[2], delta;

    amp[0] = AMP0;
    amp[1] = AMP1;

    for(j = 0; j < data_num; j++) {
        //transmit command action to h8
        while (!h8.writeable());
        h8.putc('B');
        //get ack
        while (!h8.readable());
        h8.getc();

        //gyro data receive
        for (i = 0; i < 2; i++) {
            gyro_data[i] = read_ii(&h8);
            delta = (float)((int)gyro_data[i] - gyro_newtral[i]) * amp[i];
            servo_data_work[i] = servo_data[j][i + 4];
            servo_data[j][i + 4] = servo_data[j][i + 4] - (int)delta;
        }
        //servo data send
        for (i = 0; i < 12; i++) {
            print_ii(&h8, servo_data[j][i]);
        }

        for (i = 0; i < 2; i++) {
            servo_data[j][i + 4] = servo_data_work[i];
        }

    }
}

//Play motion data with file reading line by line.
//This function does not returns servo_data.
void play_motion3(char filename[32], int gyro_newtral[2])
{
    int     i;
    int servo_data[12];
    unsigned int gyro_data[2];
    float amp[2], delta;
    char    buff[256], fields[32][64];

    FILE    *fp;

    if ((fp = fopen(filename , "rt")) == NULL) {
        printf("Motion data file open error!(play_motion2)\n");
        return;
    }

    amp[0] = AMP0;
    amp[1] = AMP1;

    while (fgets(buff, 256, fp) != NULL) {
        devide_into_field(buff, "," , fields);
        for(i = 0; i < 12; i++)
            servo_data[i] = atoi(fields[i]);

        //transmit command action to h8
        while (!h8.writeable());
        h8.putc('B');
        //get ack
        while (!h8.readable());
        h8.getc();

        //gyro data receive
        for (i = 0; i < 2; i++) {
            gyro_data[i] = read_ii(&h8);
            delta = (float)((int)gyro_data[i] - gyro_newtral[i]) * amp[i];
            servo_data[i + 4] = servo_data[i + 4] - (int)delta;
        }

        //servo data send
        for (i = 0; i < 12; i++) {
            print_ii(&h8, servo_data[i]);
        }
    }
    fclose(fp);
    return;
}
//******************************************************************************
void gyro_test(int gyro_newtral[2])
{
    int     i;
    int servo_data[12], servo_data_work[2];
    static int first_flg = 1;
    unsigned int gyro_data[2];
    unsigned char work_c;
    float amp[2];
    static float delta[2];

    printf("Gyro testing mode activated. Hit 0 to exit.\n");

    amp[0] = AMP0;
    amp[1] = AMP1;

    if(first_flg == 1) {
        first_flg = 0;
        delta[0] = 0.0;
        delta[1] = 0.0;
    }

    //get present servo data
    //transmit command action to h8
    while (!h8.writeable());
    h8.putc('C');
    //get ack
    while (!h8.readable());
    work_c = h8.getc();

    //Servo data receive
    for (i = 0; i < 12; i++) {
        servo_data[i] = read_ii(&h8);
        printf(" %d,",servo_data[i]);
    }
    printf("\n");
    //Push original data without gyro feedback.
    servo_data_work[0] = servo_data[4];
    servo_data_work[1] = servo_data[5];

    while(1) {
        if (pc.readable()) {
            work_c = pc.getc();
            if(work_c == '0') break;
        }

        //transmit command action to h8
        while (!h8.writeable());
        h8.putc('B');
        //get ack
        while (!h8.readable());
        work_c = h8.getc();

        //gyro data receive
        for (i = 0; i < 2; i++) {
            gyro_data[i] = read_ii(&h8);
            //delta[i] = delta[i] + (float)((int)gyro_data[i] - gyro_newtral[i]) * amp[i];
            delta[i] = (float)((int)gyro_data[i] - gyro_newtral[i]) * amp[i];
            servo_data[i + 4] = servo_data_work[i] - (int)delta[i];
            //delta[i] = delta[i] * dec_rate[i];
        }
        //servo data send
        for (i = 0; i < 12; i++) {
            print_ii(&h8, servo_data[i]);
        }
    }
    return;
}
