#include "mbed.h"
#include "pen4v3b.h"

extern DigitalOut led1;
extern DigitalOut led2;
extern DigitalOut led3;
extern DigitalOut led4;

extern DigitalIn pwm_status;
extern AnalogIn Battery;

extern PwmOut gyro_ref_pwm;
extern Timer timer;

extern Serial pc;
extern Serial xbee;
extern Serial h8;

unsigned int read_ii(Serial *port) {
    unsigned int    data;
    unsigned int    data_L, data_H;

    while (!port->readable());
    data_H = port->getc();

    while (!port->readable());
    data_L = port->getc();

    data = data_H * 256 + data_L;
    return(data);
}

void print_ii(Serial *port, unsigned int data) {
    unsigned int    ii;
    unsigned char    data_L, data_H;

    if (data > 255) {
        ii = data / 256;
        data_H = (unsigned char)ii;
    } else {
        ii = 0;
        data_H = 0;
    }

    ii = data - ii * 256;
    data_L = (unsigned char)ii;

    while (!port->writeable());
    port->putc(data_H);

    while (!port->writeable());
    port->putc(data_L);

    return;
}

void sio_comm1(unsigned char command, struct SIO *sio) {
    unsigned char   i, work_c, command2;

    //Gyro data receive
    if (command == 'A') {
        //Send back ack
        while (!pc.writeable());
        pc.putc(command);

        //transmit command action to h8
        while (!h8.writeable());
        h8.putc(command);
        //get ack
        while (!h8.readable());
        work_c = h8.getc();

        //gyro data receive
        for (i = 0; i < 2; i++) {
            sio->gyro_data[i] = read_ii(&h8);
        }
        //gyro data send
        for (i = 0; i < 2; i++) {
            print_ii(&pc, sio->gyro_data[i]);
        }
    }

    //Transmit 'B' command to No.2 sub CPU.
    //get angle data for the arms joints.
    if (command == 'B') {
        //Send back ack
        while (!pc.writeable());
        pc.putc(command);

        //transmit command action to h8
        while (!h8.writeable());
        h8.putc(command);
        //get ack
        while (!h8.readable());
        work_c = h8.getc();

        //gyro data receive
        for (i = 0; i < 2; i++) {
            sio->gyro_data[i] = read_ii(&h8);
        }
        //gyro data send
        for (i = 0; i < 2; i++) {
            print_ii(&pc, sio->gyro_data[i]);
        }

        //servo data receive
        for (i = 0; i < 12; i++) {
            sio->servo_data[i] = read_ii(&pc);
        }
        //servo data send
        for (i = 0; i < 12; i++) {
            print_ii(&h8, sio->servo_data[i]);
        }
    }

    //Servo data receive
    if (command == 'C') {
        //Send back ack
        while (!pc.writeable());
        pc.putc(command);

        //transmit command action to h8
        while (!h8.writeable());
        h8.putc(command);
        //get ack
        while (!h8.readable());
        work_c = h8.getc();

        //Servo data receive
        for (i = 0; i < 12; i++) {
            sio->servo_data[i] = read_ii(&h8);
        }
        //Servo data send
        for (i = 0; i < 12; i++) {
            print_ii(&pc, sio->servo_data[i]);
        }
    }

    //Transmit 'K' command to No.2 sub CPU.
    //RC servo on/off command
    if (command == 'K') {
        //Send back ack
        while (!pc.writeable());
        pc.putc(command);

        //get second command from PC
        while (!pc.readable());
        command2 = pc.getc();

        //transmit "K" command action to h8
        //write K command
        while (!h8.writeable());
        h8.putc(command);

        //get ack
        while (!h8.readable());
        work_c = h8.getc();

        if (work_c == 'K') {
            //write servo switch value
            while (!h8.writeable());
            h8.putc(command2);
        }
    }

    //Battery voltage report
    if (command == 'M') {
        //Send back ack
        while (!pc.writeable());
        pc.putc(command);

        while (!pc.writeable());
        sio->Battery = Battery * 3.3 * 3.0;
        pc.printf("%5.1f\r\n",sio->Battery);
    }

    //functional evaluation
    if (command == '9') {
    }
}

